import React, { Fragment, useState, useEffect, useCallback, createRef, useRef } from 'react';
import { View, Text, Alert, FlatList, StyleSheet, TouchableOpacity } from 'react-native';
import Styled from 'styled-components/native';
import Header from '~/Components/Layouts/Header';
import { Search, SearchIcon, Line, TextInput, checkFilter, filterValue, ListStyle, DetailIcon, RightArrow, CreateButton, SearchTitle } from '~/Components/Listing/Search';
import Checkbox from '~/Components/UIControls/Checkbox';
import Theme from '~/Themes';
import BottomTabBarLayout from '~/Components/Layouts/BottomTabBarLayout';
import session from '~/Utils/session';
import SearchList, { SearchListContainer } from '~/Components/Listing/SearchList';
import { Selectbox } from '~/Components/UIControls/Selectbox';
import EmptyView from '~/Components/Listing/EmptyView';
import EntitySelect from '~/Components/EntitySelect';
import { useToast } from '~/Components/UIControls/Message';
import { DateTimePicker } from '~/Components/UIControls/DateTimePicker';
import Input from '~/Components/UIControls/Input';
import Form from '~/Components/Form';
import { Util } from '~/Utils/utils';
import { useAuth } from '~/Components/Hooks';
import Switch from '~/Components/UIControls/Switch';

const style = StyleSheet.create({
    value : {
        width: 60,
        height: 60,
        backgroundColor: '#f5f5f5',
        borderRadius: 30,
        alignItems:'center',
        justifyContent:'center',
        marginRight: 20,
    }
});

// list item
const Item = ({item, index, onPress=()=>{}}) => {
    const zebra = index % 2 == 0 ? true : false;
    return (
        <TouchableOpacity onPress={onPress} style={{
            ...ListStyle.itemContainer, 
            borderTopWidth: 0, 
            backgroundColor: (zebra?'#fdfdfd':'#fff'),
        }}>
            <View style={{...ListStyle.vertical}}>
                <View style={[ListStyle.horizon]}>
                    <Text style={{...ListStyle.rounded}}>{Util.get(item, "FacilityTypeName")}</Text>
                    <Text>&nbsp;&nbsp;</Text>
                    <Text style={{...ListStyle.textMedium}}>
                        {Util.get(item, "MonitoringPointName")}
                    </Text>
                </View>
                <Text style={{...ListStyle.textLight}}>
                    {Util.get(item, "CreatedDateTime").replace('T', ' ')}
                </Text>
            </View>
            <View style={style.value}>
                <Text>
                {item.CurrentValue==0?'경보해제':'경보발생'}
                </Text>
            </View>
        </TouchableOpacity>
    )
}

// list and search
const Screen = (props) => {
    const navigation = props.navigation;
    const [filters, setFilters] = useState({});
    const [list, setList] = useState(null);
    const [page, setPage] = useState(1);
    const [refreshing, setRefreshing] = useState(false);
    const pages = useRef(10);
    const { auth, userinfo } = useAuth(navigation);
    const { showMessage } = useToast();

    // load session when mount...
    useEffect(() => {
    }, []);

    // search events...
    const start = () => {
        setRefreshing(true);
    }
    const success = (data, append) => {
        pages.current = data.pages;
        //console.log(data);

        if(append && page > 1) {
            setList(list.concat(data.list));
        } else {
            setList(data.list);
        }
        setRefreshing(false);
    }
    const fail = (e) => {
        showMessage(e.message);
        setRefreshing(false);
    }

    const refreshList = () => {
        setPage('refresh-' + Math.random());
    }

    return (
        <BottomTabBarLayout>
            <SearchListContainer>
                <SearchList page={page} pages={pages.current} setPage={setPage} refreshing={refreshing}
                    data={list} renderItem={({ item, index }) => {
                        return (
                            <Item item={item} index={index} onPress={() => {
                            }} />
                        )
                    }}
                    ListEmptyComponent={<EmptyView list={list} />}
                />
            </SearchListContainer>

            <Search id="alarm_log" api={`api/app/AlarmLog?siteId=${userinfo.SiteId}`} 
                    page={page} setPage={setPage} limit={50} filters={filters} sort={{field:'CreatedDateTime', order:'desc'}}
                    onStart={start} onSuccess={success} onFailure={fail} >
                <Fragment>

                    <SearchTitle title="설비유형" />
                    <EntitySelect style={{ marginBottom:20 }} entity="BemsFacilityType" defaultLabel="설비유형" emptyLabel="전체" 
                        onChange={(item) => {
                            filterValue([filters, setFilters], 'AlarmPoint.BemsMonitoringPoint.CmFacility.FacilityTypeId', item.value, 'eq');
                        }} />

                    <SearchTitle title="설비명칭" />
                    <Input style={{ marginBottom:20 }} placeholder="설비명칭" small onChangeText={value => filterValue([filters, setFilters], 'AlarmPoint.BemsMonitoringPoint.CmFacility.Name', value, 'cn')} />

                    <SearchTitle title="관제점" />
                    <Input style={{ marginBottom:20 }} placeholder="관제점" small onChangeText={value => filterValue([filters, setFilters], 'AlarmPoint.BemsMonitoringPoint.Name', value, 'cn')} />

                    <SearchTitle title="날짜" />
                    <DateTimePicker style={{ marginBottom:10 }} placeholder="시작일" useClear={true} onChange={(newDate) => {
                                filterValue([filters, setFilters], 'CreatedDateTime', Util.toDateString(newDate) + ' 00:00:00.000', 'ge');}} />
                    <DateTimePicker style={{ marginBottom:20 }} placeholder="종료일" useClear={true} onChange={(newDate) => {
                                filterValue([filters, setFilters], 'CreatedDateTime', Util.toDateString(newDate) + ' 23:59:59.998', 'le');}} />

                    <SearchTitle title="경보상태" />
                    <Selectbox style={{ marginBottom:20 }} defaultLabel="경보상태" emptyLabel="전체"
                        items={[{label:'경보발생', value:true}, {label:'경보해제', value:false}]}
                        onChange={item => {
                            filterValue([filters, setFilters], 'CurrentValue', null, 'ne');
                            filterValue([filters, setFilters], 'CurrentValue', null, 'eq');

                            if(item.value == true) {
                                filterValue([filters, setFilters], 'CurrentValue', 0, 'ne');
                            } else if(item.value == false) {
                                filterValue([filters, setFilters], 'CurrentValue', 0, 'eq');
                            }
                        }} />
                </Fragment>
            </Search>
        </BottomTabBarLayout> 
    );
};

// header
Screen.navigationOptions = ({navigation}) => { 
    return { 
        header: () => <Header 
            navigation={navigation} 
            title="경보이력" 
            buttons={<SearchIcon id="alarm_log" />}
        /> 
    } 
};

export default Screen;